unit MainDlg;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ComCtrls, StdCtrls, ExtCtrls;

type
  TMainForm = class(TForm)
    PageControl: TPageControl;
    FileTabSheet: TTabSheet;
    AutostartTabSheet: TTabSheet;
    AddButton: TButton;
    RemoveButton: TButton;
    AutostartLabel: TLabel;
    BeerTabSheet: TTabSheet;
    KillButton: TButton;
    InstallButton: TButton;
    RunButton: TButton;
    GetDataButton: TButton;
    SendButton: TButton;
    Memo: TMemo;
    Panel1: TPanel;
    ProgressBar: TProgressBar;
    FileStatusLabel: TLabel;
    procedure SendButtonClick(Sender: TObject);
    procedure GetDataButtonClick(Sender: TObject);
    procedure InstallButtonClick(Sender: TObject);
    procedure RunButtonClick(Sender: TObject);
    procedure KillButtonClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure RemoveButtonClick(Sender: TObject);
    procedure AddButtonClick(Sender: TObject);
  private
    procedure UpdateAutostartStatus;
    procedure UpdateFileStatus(const StatusMessage : string);
    procedure RunDeviceApplication(const FileName,CommandLine : string);
    function  DeviceFileExists(const FileName : string) : Boolean;
    procedure CopyFileToDevice(const FileName,DeviceDirectory : string);
    procedure CopyFileFromDevice(const DeviceFileName,PCDirectory : string);
    procedure FileOperationsProgressInfo(Sender : TObject;
                                         ProgrPerc,ProgrAbs,ProgrTot : Integer;
                                         FileName : string);
    procedure CreateShortcut(const Shortcut,Target : string);
  public
  end;

var
  MainForm: TMainForm;

implementation

uses
  Autostart, CETools, StrUtils;

{$R *.dfm}

procedure TMainForm.UpdateAutostartStatus;
begin
  if TAutostart.OnList then begin
    AutostartLabel.Caption := 'On Autostart list';
  end else begin
    AutostartLabel.Caption := 'Not on Autostart list';
  end;
end;

procedure TMainForm.UpdateFileStatus(const StatusMessage : string);
begin
  FileStatusLabel.Caption := StatusMessage;
  Application.ProcessMessages;
end;

procedure TMainForm.RunDeviceApplication(const FileName,CommandLine : string);
  var
    CERunApp : TCERunApp;
begin
  CERunApp := TCERunApp.Create(nil);

  try
    CERunApp.ApplicationName := FileName;
    CERunApp.CommandLine     := CommandLine;

    if CERunApp.Run then begin
      UpdateFileStatus('Ran application ' + FileName + ' on device');
    end else begin
      UpdateFileStatus('Unable to run application ' + FileName + ' on device');
    end;
  finally
    CERunApp.Free;
  end;
end;

function TMainForm.DeviceFileExists(const FileName : string) : Boolean;
  var
    CEFindFile : TCEFindFile;
begin
  Result     := False;
  CEFindFile := TCEFindFile.Create(nil);

  try
    CEFindFile.Filter     := ExtractFileName(FileName);
    CEFindFile.Attributes := [];
    CEFindFile.Directory  := ExtractFilePath(FileName);
    CEFindFile.Recurse    := false;

    if CEFindFile.Execute then begin
      // Can get an empty string returned, so check
      Result := (CEFindFile.Files.Count > 0) and (CEFindFile.Files[0] > '');
    end;
  finally
    CeFindFile.Free;
  end;
end;

procedure TMainForm.CopyFileToDevice(const FileName,DeviceDirectory : string);
  var
    CEFileOperations : TCEFileOperations;
    TempDirectory    : string;
begin
  if not FileExists(FileName) then begin
    Exit;
  end;

  CEFileOperations := TCEFileOperations.Create(nil);

  try
    CEFileOperations.OnProgressInfo         := FileOperationsProgressInfo;
    CEFileOperations.FormProperties.Visible := False;

    // Try to create directory in case it is needed. It's OK to let it fail
    // Need to remove the trailing backslash before creating the directory though
    TempDirectory := LeftStr(DeviceDirectory,Length(DeviceDirectory) - 1);

    if CEFileOperations.CreateDirectory(TempDirectory) then begin
      UpdateFileStatus('Created directory ' + TempDirectory);
    end;

    // Copy file across
    CEFileOperations.SourceFilename := FileName;
    CEFileOperations.TargetFilename := DeviceDirectory + ExtractFileName(FileName);
    CEFileOperations.Operation      := coCopyDesktopToCE;

    if CEFileOperations.Execute then begin
      UpdateFileStatus('Copied ' + FileName + ' to device');
    end else begin
      UpdateFileStatus('Unable to copy ' + FileName + ' to device');
    end
  finally
    CEFileOperations.Free;
    ProgressBar.Position := 0;
  end;
end;

procedure TMainForm.CopyFileFromDevice(const DeviceFileName,PCDirectory : string);
  var
    CEFileOperations : TCEFileOperations;
begin
  if not DeviceFileExists(DeviceFileName) then begin
    Exit;
  end;

  CEFileOperations := TCEFileOperations.Create(nil);

  try
    CEFileOperations.OnProgressInfo         := FileOperationsProgressInfo;
    CEFileOperations.FormProperties.Visible := False;

    // Copy file across
    CEFileOperations.SourceFilename := DeviceFileName;
    CEFileOperations.TargetFilename := PCDirectory + ExtractFileName(DeviceFileName);
    CEFileOperations.Operation      := coCopyCEToDesktop;

    if CEFileOperations.Execute then begin
      UpdateFileStatus('Copied ' + DeviceFileName + ' to PC');
    end else begin
      UpdateFileStatus('Unable to copy ' + DeviceFileName + ' to PC');
    end
  finally
    CEFileOperations.Free;
    ProgressBar.Position := 0;
  end;
end;

procedure TMainForm.FileOperationsProgressInfo(Sender : TObject;
                                               ProgrPerc,ProgrAbs,ProgrTot : Integer;
                                               FileName : String);
begin
  if ProgressBar <> nil then begin
    ProgressBar.Max      := 100;
    ProgressBar.Position := ProgrPerc;
  end;
end;

procedure TMainForm.CreateShortcut(const Shortcut,Target : string);
  var
    CEShortCut : TCEShortCut;
begin
  CEShortCut := TCEShortCut.Create(nil);

  try
    // Path might be different on SmartPhones - so check!
    CEShortCut.ShortCut := '\Windows\Start Menu\' + Shortcut + '.lnk';
    CEShortCut.Target   := '"' + Target + '"';
    CEShortCut.Open;

    if CEShortCut.CreateShortCut then begin
      UpdateFileStatus('Created shortcut ' + Shortcut);
    end;

    CEShortCut.Close;
  finally
    CEShortCut.Free;
  end;
end;


procedure TMainForm.FormShow(Sender: TObject);
begin
  UpdateAutostartStatus;
  PageControl.ActivePage := AutostartTabSheet;
end;

procedure TMainForm.AddButtonClick(Sender: TObject);
begin
  TAutostart.Add;
  UpdateAutostartStatus;
end;

procedure TMainForm.RemoveButtonClick(Sender: TObject);
begin
  TAutostart.Remove;
  UpdateAutostartStatus;
end;

procedure TMainForm.InstallButtonClick(Sender: TObject);
begin
  CopyFileToDevice('C:\Conferences\SDN\2005 Sept\Source\BeerTaster\bin\Debug\BeerTaster.exe',
                   '\Program Files\BeerTaster\');
  CopyFileToDevice('C:\Conferences\SDN\2005 Sept\Source\AppKiller\bin\Debug\AppKiller.exe',
                   '\Program Files\BeerTaster\');
  CreateShortcut('BeerTaster','\Program Files\BeerTaster\BeerTaster.exe');
end;

procedure TMainForm.RunButtonClick(Sender: TObject);
begin
  RunDeviceApplication('\Program Files\BeerTaster\BeerTaster.exe','');
end;

procedure TMainForm.KillButtonClick(Sender: TObject);
begin
  RunDeviceApplication('\Program Files\BeerTaster\AppKiller.exe','BeerTaster.exe');
end;

procedure TMainForm.GetDataButtonClick(Sender: TObject);
begin
  // Make sure application on device is closed so latest data is saved
  RunDeviceApplication('\Program Files\BeerTaster\AppKiller.exe','BeerTaster.exe');
  // Copy data file to PC
  CopyFileFromDevice('\Program Files\BeerTaster\BeerData.xml',ExtractFilePath(Application.ExeName));
  // Display data
  Memo.Lines.LoadFromFile(ExtractFilePath(Application.ExeName) + 'BeerData.xml');
end;

procedure TMainForm.SendButtonClick(Sender: TObject);
begin
  Memo.Lines.SaveToFile(ExtractFilePath(Application.ExeName) + 'BeerData.xml');
  CopyFileToDevice(ExtractFilePath(Application.ExeName) + 'BeerData.xml',
                   '\Program Files\BeerTaster\');
end;

end.
